# -*- coding: utf8 -*-
import requests
import json
import hmac, time, hashlib
import re
import datetime
import os
from tencentcloud.cls.sign import signature
from six.moves.urllib.parse import quote, unquote, urlparse, urlencode
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.cdn.v20180606 import cdn_client, models

secret_id = os.environ.get('secret_id')
secret_key = os.environ.get('secret_key')
LOGSETID = os.environ.get('logset_id')
TOPICIDS = os.environ.get('topic_ids')
CLSDAYNUM = os.environ.get('cls_day_num')
HOSTS = os.environ.get('hosts').split(";")
cls_domain = os.environ.get('cls_domain')
cos_bucket_name = os.environ.get('cos_bucket_name')

def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i:i + n]

def product_days():
    utc_dt = datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc)
    bj_dt = utc_dt.astimezone(datetime.timezone(datetime.timedelta(hours=8)))
    days = []
    for i in range(0, int(CLSDAYNUM)):
        before = bj_dt - datetime.timedelta(days=i)
        days.append(before.strftime("%Y-%m-%d %H:%M:%S"))
    return days


def send_cls_search(start_time, end_time, cos_url, context=""):
    cls_result = {}
    cos_cls_url = cos_url.replace('/','\/') 
    bucket_name = 'bucketName:{0} AND '.format(cos_bucket_name)
    from_cdn_access = 'logSourceType:CDN AND ' 
    query_string = bucket_name + from_cdn_access + r'''reqPath:\"%s*''' % cos_cls_url
    print(query_string)
    params = {
        'start_time': start_time,
        'end_time': end_time,
        'logset_id': LOGSETID,
        "query_string": query_string,
        'topic_ids': TOPICIDS,
        "limit": 100,
        "sort": "desc",
    }
    if context:
        params["context"] = context
    Authorization = signature(secret_id=secret_id, secret_key=secret_key, method='GET', path='/searchlog',
                              params=params, headers={'Host': cls_domain, 'User-Agent': 'AuthSDK'},
                              expire=300)

    header = {
        "Content-Type": "application/json",
        "Authorization": Authorization
    }
    search_url = "http://{0}/searchlog".format(cls_domain)
    res = requests.get(search_url, headers=header, params=params)
    url_set = set()
    request_id_list = []
    if res.status_code == 200:
        res_json = res.json()
        listover = res_json["listover"]  # 搜索结果是否已经全部返回
        context = res_json["context"]  # 获取更多检索结果的游标
        results = res_json["results"]
        for result in results:
            content = json.loads(result["content"])
            request_id_list.append(content["requestId"])
            requestUri = content["requestUri"]
            requestUri = str.replace(requestUri, "\"", "")
            re_url = re.search(cos_url + '.*\s', requestUri)
            if re_url:
                url_set.add(re_url.group().replace(" ", ""))
            else:
                print("parse url not match: {0},{1}".format(cos_url + '.*\s', requestUri))
        cls_result = {
            "listover": listover,
            "context": context,
            "url_set": url_set,
            "request_id": request_id_list
        }
    else:
        print("send cls request error: %s" % res.content)
    return cls_result


def flush_path(cos_path):
    days = product_days()
    print("---------------CLS SEARCH START----------------")
    path_set = set()
    for i in range(0, len(days) - 1):
        # 过滤七天的日志，找到所有回源url
        end_time = days[i]
        start_time = days[i + 1]
        search_result = send_cls_search(start_time, end_time, cos_path)
        path_set = path_set.union(search_result["url_set"])
        while not search_result["listover"]:
            search_result = send_cls_search(start_time, end_time, cos_path, search_result["context"])
            path_set.union(search_result["url_set"])
    print("cls search path set: ")
    print(path_set)
    print("---------------CLS SEARCH END----------------")
    cred = credential.Credential(secret_id, secret_key)
    httpProfile = HttpProfile()
    httpProfile.endpoint = "cdn.tencentcloudapi.com"
    print("---------------FLUSH CDN URL START----------------")
    urls = []
    for host in HOSTS:
        for path in path_set:
            urls.append("https://" + host + path)
        if len(path_set) == 0:
            urls.append("https://" + host + cos_path)
    clientProfile = ClientProfile()
    clientProfile.httpProfile = httpProfile
    client = cdn_client.CdnClient(cred, "", clientProfile)
    req = models.PurgeUrlsCacheRequest()

    for chunk_urls in chunks(urls, 1000):
        params = {
            "Urls": chunk_urls
        }
        # print("send PurgeUrlsCache urls: %s " % json.dumps(params))
        req.from_json_string(json.dumps(params))
        resp = client.PurgeUrlsCache(req)
        # print("recv response=: %s" % resp.to_json_string())
    print("---------------FLUSH CDN URL END----------------")

def main_handler(event, context):
    records = event.get("Records", [])
    print("Records: ", json.dumps(records))
    delete_paths = []
    for record in records:
        url = record["cos"]["cosObject"]["url"]
        parse_url = urlparse(url)
        delete_paths.append(parse_url.path)

    for delete_path in delete_paths:
        flush_path(delete_path)

    return ("finish url flush: %s" % json.dumps(delete_paths))
