# -*- coding: utf8 -*-
import hmac, time, hashlib
from six.moves.urllib.parse import quote, unquote, urlparse, urlencode

def signature(secret_id, secret_key, method='GET', path='/', params={}, headers={}, expire=120):
    # reserved keywords in headers urlencode are -_.~, notice that / should be encoded and space should not be encoded to plus sign(+)
    filt_headers = dict((k.lower(), quote(headers[k].encode('utf-8'), '-_.~')) \
                        for k in headers if
                        k.lower() == 'content-type' or k.lower() == 'content-md5' or k.lower() == 'host' or k[
                            0].lower() == 'x')
    uri_params = dict((k.lower(), params[k]) for k in params)
    format_str = u"{method}\n{host}\n{params}\n{headers}\n".format(
        method=method.lower(),
        host=path,
        params=urlencode(sorted(uri_params.items())),
        headers='&'.join(map(lambda tupl: "%s=%s" % (tupl[0], tupl[1]), sorted(filt_headers.items())))
    )
    start_sign_time = int(time.time())
    sign_time = "{bg_time};{ed_time}".format(bg_time=start_sign_time - 60, ed_time=start_sign_time + expire)
    # sign_time = "1588236945;1588237305"
    sha1 = hashlib.sha1()
    sha1.update(format_str.encode('utf-8'))

    str_to_sign = "sha1\n{time}\n{sha1}\n".format(time=sign_time, sha1=sha1.hexdigest())
    sign_key = hmac.new(secret_key.encode('utf-8'), sign_time.encode('utf-8'), hashlib.sha1).hexdigest()
    sign = hmac.new(sign_key.encode('utf-8'), str_to_sign.encode('utf-8'), hashlib.sha1).hexdigest()
    sign_tpl = "q-sign-algorithm=sha1&q-ak={ak}&q-sign-time={sign_time}&q-key-time={key_time}&q-header-list={headers}&q-url-param-list={params}&q-signature={sign}"

    return sign_tpl.format(
        ak=secret_id,
        sign_time=sign_time,
        key_time=sign_time,
        params=';'.join(sorted(map(lambda k: k.lower(), uri_params.keys()))),
        headers=';'.join(sorted(filt_headers.keys())),
        sign=sign
    )